@@ -80,6 +80,7 @@ from bzrlib.plugins.svn import (
     )
 from bzrlib.plugins.svn.config import (
     BranchConfig,
+    SvnBranchStack,
     )
 from bzrlib.plugins.svn.errors import (
     NotSvnBranchPath,
@@ -221,8 +222,7 @@ class SvnBranch(ForeignBranch):
         self._format = SvnBranchFormat()
         self.layout = self.repository.get_layout()
         self._branch_path = branch_path.strip("/")
-        self.base = urlutils.join(self.repository.base,
-                        self._branch_path).rstrip("/")
+        self.base = urlutils.join(self.repository.base, urlutils.escape(self._branch_path)).rstrip("/")
         super(SvnBranch, self).__init__(mapping)
         self._lock_mode = None
         self._lock_count = 0
@@ -242,6 +242,9 @@ class SvnBranch(ForeignBranch):
             raise NotBranchError(branch_path)
         if type not in ('branch', 'tag') or ip != '':
             raise NotBranchError(branch_path)
+        self.name = self.layout.get_branch_name(branch_path)
+        if not isinstance(self.name, unicode):
+            raise TypeError(self.name)
 
     @property
     def control_url(self):
@@ -451,8 +454,14 @@ class SvnBranch(ForeignBranch):
     def get_config(self):
         return BranchConfig(self.base, self.repository.uuid)
 
+    def get_config_stack(self):
+        return SvnBranchStack(self.base, self.repository.uuid)
+
     def get_append_revisions_only(self):
-        return self.get_config().get_append_revisions_only()
+        value = self.get_config_stack().get('append_revisions_only')
+        if value is None:
+            value = True
+        return value
 
     def _get_nick(self, local=False, possible_master_transports=None):
         """Find the nick name for this branch.
@@ -510,7 +519,7 @@ class SvnBranch(ForeignBranch):
             else:
                 base_foreign_info = self.lookup_bzr_revision_id(rev.parent_ids[0])
             interrepo.push_single_revision(
-                self.get_branch_path(), self.get_config(), rev,
+                self.get_branch_path(), self.get_config_stack(), rev,
                 push_metadata=True, root_action=("replace", self.get_revnum()),
                 base_foreign_info=base_foreign_info)
         self._clear_cached_state()
@@ -608,7 +617,7 @@ class SvnBranch(ForeignBranch):
 
     def get_push_merged_revisions(self):
         return (self.layout.push_merged_revisions(self.project) and
-                self.get_config().get_push_merged_revisions())
+                self.get_config_stack().get('push_merged_revisions'))
 
     def import_last_revision_info(self, source_repo, revno, revid, lossy=False):
         interrepo = InterToSvnRepository(source_repo, self.repository)
@@ -616,7 +625,7 @@ class SvnBranch(ForeignBranch):
         revidmap = interrepo.push_todo(self.last_revision(),
             base_revmeta.metarev.get_foreign_revid(),
             base_mapping, revid, self.layout,
-            self.project, self.get_branch_path(), self.get_config(),
+            self.project, self.get_branch_path(), self.get_config_stack(),
             push_merged=self.get_push_merged_revisions(),
             overwrite=False, push_metadata=not lossy,
             append_revisions_only=True)
@@ -1056,7 +1065,7 @@ class InterToSvnBranch(InterBranch):
         base_revmeta, base_mapping = self.target.last_revmeta(skip_hidden=False)
         try:
             revidmap = interrepo.push_branch(self.target.get_branch_path(),
-                    self.target.get_config(), old_last_revid,
+                    self.target.get_config_stack(), old_last_revid,
                     base_revmeta.metarev.get_foreign_revid(),
                     base_mapping,
                     stop_revision=stop_revision, overwrite=overwrite,
@@ -1091,11 +1100,6 @@ class InterToSvnBranch(InterBranch):
                 new_last_revid,
                 dict([(k, v[0]) for (k, v) in revid_map.iteritems()]))
 
-    def lossy_push(self, stop_revision=None):
-        """See InterBranch.lossy_push()."""
-        # For compatibility with bzr < 2.4
-        return self.push(lossy=True, stop_revision=stop_revision)
-
     def fetch(self, stop_revision=None, fetch_tags=None, find_ghosts=False,
             limit=None, exclude_non_mainline=None):
         """Fetch into a subversion repository."""
