@@ -29,6 +29,8 @@ from bzrlib.option import (
     Option,
     )
 
+from bzrlib.plugins.svn import gettext
+
 
 def get_layout(layoutname):
     """Parse layout name and return a layout.
@@ -43,7 +45,8 @@ def get_layout(layoutname):
     try:
         ret = layout_registry.get(layoutname)()
     except KeyError:
-        raise BzrCommandError('No such repository layout %r' % layoutname)
+        raise BzrCommandError(gettext('No such repository layout %r') %
+                layoutname)
     return ret
 
 
@@ -99,6 +102,7 @@ class cmd_svn_import(Command):
             BzrCommandError,
             NoRepositoryPresent,
             )
+        from bzrlib.plugins.svn import gettext
         from bzrlib.plugins.svn.convert import convert_repository
         from bzrlib.plugins.svn.remote import SvnRemoteAccess
         from bzrlib.plugins.svn.repository import SvnRepository
@@ -127,14 +131,15 @@ class cmd_svn_import(Command):
 
         if not (isinstance(from_dir, SvnRemoteAccess) or
                 isinstance(from_dir, SvnCheckout)):
-            raise BzrCommandError("Source repository is not a Subversion repository.")
+            raise BzrCommandError(gettext(
+                "Source repository is not a Subversion repository."))
 
         try:
             from_repos = from_dir.open_repository()
         except NoRepositoryPresent, e:
             if prefix is not None:
-                raise BzrCommandError("Path inside repository specified "
-                                      "and --prefix specified")
+                raise BzrCommandError(gettext("Path inside repository specified "
+                                      "and --prefix specified"))
             from_repos = from_dir.find_repository(_ignore_branch_path=True)
             assert from_dir.root_transport.base.startswith(from_repos.base)
             prefix = from_dir.root_transport.base[len(from_repos.base):].strip("/")
@@ -142,7 +147,7 @@ class cmd_svn_import(Command):
 
         if not isinstance(from_repos, SvnRepository):
             raise BzrCommandError(
-                    "Not a Subversion repository: %s" % from_location)
+                    gettext("Not a Subversion repository: %s") % from_location)
 
         if until is None:
             to_revnum = from_repos.get_latest_revnum()
@@ -158,16 +163,16 @@ class cmd_svn_import(Command):
                     overall_layout = layout
                 prefix = prefix.strip("/") + "/"
                 if overall_layout.is_branch(prefix):
-                    raise BzrCommandError("%s appears to contain a branch. "
-                            "For individual branches, use 'bzr branch'." %
+                    raise BzrCommandError(gettext("%s appears to contain a branch. "
+                            "For individual branches, use 'bzr branch'.") %
                             from_location)
                 # FIXME: Hint about is_tag()
                 elif overall_layout.is_branch_parent(prefix):
-                    self.outf.write("Importing branches with prefix /%s\n" %
-                        urlutils.unescape_for_display(prefix, self.outf.encoding))
+                    self.outf.write(gettext(gettext("Importing branches with prefix %s\n")) %
+                        ("/" + urlutils.unescape_for_display(prefix, self.outf.encoding)))
                 else:
-                    raise BzrCommandError("The specified path is inside a branch. "
-                        "Specify a different URL or a different repository layout (see also 'bzr help svn-layout').")
+                    raise BzrCommandError(gettext("The specified path is inside a branch. "
+                        "Specify a different URL or a different repository layout (see also 'bzr help svn-layout')."))
 
             if (prefix is not None and
                 from_repos.transport.check_path(prefix, to_revnum) == NODE_NONE):
@@ -179,7 +184,7 @@ class cmd_svn_import(Command):
                     return False
                 return True
 
-            trace.note("Using repository layout: %s",
+            trace.note(gettext("Using repository layout: %s"),
                        layout or from_repos.get_layout())
             convert_repository(from_repos, to_location, layout,
                 not standalone, trees, all, format=format,
@@ -189,8 +194,9 @@ class cmd_svn_import(Command):
             if tmp_repos is not None:
                 osutils.rmtree(tmp_repos)
             if not trees:
-                trace.note("Use 'bzr checkout' to create a working tree in "
-                           "the newly created branches.")
+                trace.note(
+                    gettext("Use 'bzr checkout' to create a working tree in "
+                            "the newly created branches."))
         finally:
             from_repos.unlock()
 
@@ -210,6 +216,7 @@ class cmd_svn_layout(Command):
             urlutils,
             )
         from bzrlib.branch import Branch
+        from bzrlib.plugins.svn import gettext
         from bzrlib.repository import Repository
         from bzrlib.plugins.svn import errors as bzrsvn_errors
 
@@ -221,32 +228,33 @@ class cmd_svn_layout(Command):
             branch = None
         if getattr(repos, "uuid", None) is None:
             raise errors.BzrCommandError(
-                "Not a Subversion branch or repository.")
+                gettext("Not a Subversion branch or repository."))
         layout = repos.get_layout()
-        self.outf.write("Repository root: %s\n" % repos.base)
-        self.outf.write("Layout: %s\n" % str(layout))
+        self.outf.write(gettext("Repository root: %s\n") % repos.base)
+        self.outf.write(gettext("Layout: %s\n") % str(layout))
         if branch is not None:
-            self.outf.write("Branch path: %s\n" % branch.get_branch_path())
+            self.outf.write(gettext("Branch path: %s\n") % branch.get_branch_path())
             if branch.project:
-                self.outf.write("Project: %s\n" % branch.project)
+                self.outf.write(gettext("Project: %s\n") % branch.project)
             try:
                 test_tag_path = layout.get_tag_path("test", branch.project)
             except bzrsvn_errors.NoLayoutTagSetSupport:
-                self.outf.write("No tag support\n")
+                self.outf.write(gettext("No tag support\n"))
             else:
                 if test_tag_path:
-                    self.outf.write("Tag container directory: %s\n" %
+                    self.outf.write(gettext("Tag container directory: %s\n") %
                             urlutils.dirname(test_tag_path))
             try:
                 test_branch_path = layout.get_branch_path("test",
                     branch.project)
             except bzrsvn_errors.NoCustomBranchPaths:
-                self.outf.write("No custom branch support\n")
+                self.outf.write(gettext("No custom branch support\n"))
             else:
                 if test_branch_path:
-                    self.outf.write("Branch container directory: %s\n" %
-                            urlutils.dirname(test_branch_path))
-            self.outf.write("Push merged revisions: %s\n" %
+                    self.outf.write(
+                        gettext("Branch container directory: %s\n" %
+                            urlutils.dirname(test_branch_path)))
+            self.outf.write(gettext("Push merged revisions: %s\n") %
                     branch.get_push_merged_revisions())
 
 
@@ -267,6 +275,7 @@ class cmd_svn_branches(Command):
     def run(self, location, layout=None):
         from bzrlib import errors
         from bzrlib.bzrdir import BzrDir
+        from bzrlib.plugins.svn import gettext
         from bzrlib.plugins.svn.remote import SvnRemoteAccess
         from bzrlib.plugins.svn.workingtree import SvnCheckout
 
@@ -275,7 +284,7 @@ class cmd_svn_branches(Command):
         if not (isinstance(dir, SvnRemoteAccess) or
                 isinstance(dir, SvnCheckout)):
             raise errors.BzrCommandError(
-                "Source repository is not a Subversion repository.")
+                gettext("Source repository is not a Subversion repository."))
 
         try:
             repository = dir.open_repository()
@@ -291,9 +300,68 @@ class cmd_svn_branches(Command):
         if layout is None:
             layout = repository.get_guessed_layout()
 
-        self.outf.write("Branches:\n")
+        self.outf.write(gettext("Branches:\n"))
         for (project, path, name, has_props, revnum) in layout.get_branches(repository, revnum, prefix):
             self.outf.write("%s (%s)\n" % (path, name))
-        self.outf.write("Tags:\n")
+        self.outf.write(gettext("Tags:\n"))
         for (project, path, name, has_props, revnum) in layout.get_tags(repository, revnum, prefix):
             self.outf.write("%s (%s)\n" % (path, name))
+
+
+class cmd_fix_svn_ancestry(Command):
+    """Fix the SVN ancestry of a repository.
+
+    This will fix revisions that were imported from Subversion with older
+    versions of bzr-svn but have some incorrect metadata.
+
+    This command should be used in the bzr branch with the broken
+    imported revisions, and takes the location of the SVN repository
+    to fetch the fixed revisions from as an argument.
+    """
+
+    takes_args = ['svn_repository']
+    takes_options = [
+        'directory',
+        Option('no-reconcile', help="Don't reconcile the new repository.")]
+
+    def run(self, svn_repository, directory=".", no_reconcile=False):
+        from bzrlib.controldir import ControlDir
+        from bzrlib.repository import InterRepository, Repository
+        from bzrlib import trace
+        correct_dir = ControlDir.open_containing(svn_repository)[0]
+        correct_repo = correct_dir.find_repository()
+        repo_to_fix = Repository.open(directory)
+        revids = repo_to_fix.all_revision_ids()
+        present_revisions = correct_repo.has_revisions(revids)
+        dir_to_fix = repo_to_fix.bzrdir
+        old_repo_format = repo_to_fix._format
+        del repo_to_fix
+        trace.note("Renaming existing repository to repository.backup.")
+        dir_to_fix.control_transport.rename('repository', 'repository.backup')
+        backup_transport = dir_to_fix.control_transport.clone(
+            'repository.backup')
+        old_repo = old_repo_format.open(dir_to_fix, _found=True,
+            _override_transport=backup_transport)
+        new_repo = dir_to_fix.create_repository(
+            shared=old_repo.is_shared())
+        working_trees = old_repo.make_working_trees()
+        if working_trees is not None:
+            new_repo.set_make_working_trees(working_trees)
+        interrepo = InterRepository.get(correct_repo, new_repo)
+        revisionfinder = interrepo.get_revision_finder(True)
+        trace.note("Finding revisions to fetch from SVN")
+        for revid in present_revisions:
+            foreign_revid, mapping = correct_repo.lookup_bzr_revision_id(
+                revid)
+            revisionfinder.find_until(foreign_revid, mapping,
+                find_ghosts=False, exclude_non_mainline=False)
+        trace.note("Fetching correct SVN revisions")
+        interrepo.fetch(needed=revisionfinder.get_missing())
+        trace.note("Fetching other revisions")
+        new_repo.fetch(old_repo)
+        if not no_reconcile:
+            from bzrlib.reconcile import reconcile
+            trace.note("Reconciling new repository.")
+            reconcile(dir_to_fix)
+        trace.note('Removing backup')
+        dir_to_fix.control_transport.delete_tree('repository.backup')
