@@ -61,10 +61,7 @@ from bzrlib.inventory import (
 from bzrlib.revision import (
     NULL_REVISION,
     )
-try:
-    from bzrlib.revisiontree import InventoryRevisionTree
-except ImportError: # bzr < 2.4
-    from bzrlib.revisiontree import RevisionTree as InventoryRevisionTree
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.repository import (
     InterRepository,
     )
@@ -77,6 +74,7 @@ from bzrlib.plugins.svn import changes
 from bzrlib.plugins.svn.errors import (
     AbsentPath,
     InvalidFileName,
+    SymlinkTargetContainsNewline,
     TextChecksumMismatch,
     convert_svn_error,
     )
@@ -644,8 +642,7 @@ class FileRevisionBuildEditor(FileBuildEditor):
                     self.parent_file_id)
             ie.symlink_target = "".join(chunks)[len("link "):].decode("utf-8")
             if "\n" in ie.symlink_target:
-                raise AssertionError("bzr doesn't support newlines in symlink "
-                                     "targets yet")
+                raise SymlinkTargetContainsNewline(self.path)
             chunks = []
             text_sha1 = osutils.sha_string("")
         else:
@@ -1255,7 +1252,7 @@ class FetchRevisionFinder(object):
                     else:
                         raise
                 if (m != master_mapping and
-                    not m.is_branch_or_tag(revmeta.branch_path)):
+                    not m.is_branch_or_tag(revmeta.metarev.branch_path)):
                     continue
                 lhs_parent_revmeta = revmeta.get_lhs_parent_revmeta(m)
                 if (lhs_parent_revmeta is not None and
@@ -1269,7 +1266,7 @@ class FetchRevisionFinder(object):
                     needs_checking.append((revmeta, m))
         self.needed.extend(reversed(self.check_revmetas(needs_checking)))
 
-    def find_all(self, mapping):
+    def find_all(self, mapping, layout):
         """Find all revisions from the source repository that are not
         yet in the target repository.
 
@@ -1279,7 +1276,7 @@ class FetchRevisionFinder(object):
         pb = ui.ui_factory.nested_progress_bar()
         try:
             all_revs = self.source._revmeta_provider.iter_all_revisions(
-                    self.source.get_layout(),
+                    layout,
                     check_unusual_path=mapping.is_branch_or_tag,
                     from_revnum=from_revnum, pb=pb)
             return self.find_iter_revisions(all_revs, mapping, lambda x: False)
@@ -1606,7 +1603,8 @@ class InterFromSvnRepository(InterRepository):
                     find_ghosts=find_ghosts,
                     exclude_non_mainline=exclude_non_mainline)
         else:
-            revisionfinder.find_all(self.source.get_mapping())
+            revisionfinder.find_all(self.source.get_mapping(),
+                    self.source.get_layout())
         return revisionfinder.get_missing()
 
     def fetch(self, revision_id=None, pb=None, find_ghosts=False,
