@@ -160,8 +160,8 @@ class SvnRemoteFormat(ControlDirFormat):
         import os
         import subvertpy
         from subvertpy import repos
+        ERR_REPOS_BAD_ARGS = getattr(subvertpy, "ERR_REPOS_BAD_ARGS", 165002)
         # For subvertpy < 0.8.6
-        ERR_DIR_NOT_EMPTY = getattr(subvertpy, "ERR_DIR_NOT_EMPTY", 200011)
 
         if not isinstance(transport, LocalTransport):
             raise UninitializableOnRemoteTransports(self)
@@ -171,8 +171,10 @@ class SvnRemoteFormat(ControlDirFormat):
         try:
             repos.create(local_path)
         except subvertpy.SubversionException, (_, num):
-            if num == ERR_DIR_NOT_EMPTY:
+            if num == subvertpy.ERR_DIR_NOT_EMPTY:
                 raise errors.BzrError("Directory is not empty")
+            if num == ERR_REPOS_BAD_ARGS:
+                raise errors.AlreadyControlDirError(local_path)
             raise
         # All revision property changes
         revprop_hook = os.path.join(local_path, "hooks", "pre-revprop-change")
@@ -224,7 +226,10 @@ class SvnRemoteAccess(ControlDir):
         self.svn_root_url = _transport.get_svn_repos_root()
         self.root_url = _transport.get_repos_root()
 
-        assert self.svn_url.lower().startswith(self.svn_root_url.lower())
+        if not self.svn_url.lower().startswith(self.svn_root_url.lower()):
+            raise AssertionError("SVN URL %r does not start with root %r" %
+                (self.svn_url, self.svn_root_url))
+
         self._branch_path = urllib.unquote(self.svn_url[len(self.svn_root_url):])
 
     def break_lock(self):
@@ -287,7 +292,8 @@ class SvnRemoteAccess(ControlDir):
                         'use "bzr svn-import".')
         target_transport = get_transport(url, possible_transports)
         target_transport.ensure_base()
-        cloning_format = self.cloning_metadir()
+        require_colocated = ("branch" in target_transport.get_segment_parameters())
+        cloning_format = self.cloning_metadir(require_colocated=require_colocated)
         # Create/update the result branch
         result = cloning_format.initialize_on_transport(target_transport)
 
@@ -368,11 +374,14 @@ class SvnRemoteAccess(ControlDir):
         else:
             return SvnRepository(self, transport, self._branch_path)
 
-    def cloning_metadir(self, require_stacking=False):
+    def cloning_metadir(self, require_stacking=False, require_colocated=False):
         """Produce a metadir suitable for cloning with."""
-        return format_registry.make_bzrdir('default')
+        ret = format_registry.make_bzrdir('default')
+        if require_colocated and not ret.colocated_branches:
+            ret = format_registry.make_bzrdir('development-colo')
+        return ret
 
-    def open_workingtree(self, _unsupported=False,
+    def open_workingtree(self, unsupported=False,
             recommend_upgrade=True):
         """See ControlDir.open_workingtree().
 
@@ -437,19 +446,18 @@ class SvnRemoteAccess(ControlDir):
         layout = repos.get_layout()
         if branch_name is None and getattr(self, "_get_selected_branch", False):
             branch_name = self._get_selected_branch()
-        if branch_name is None and layout.is_branch_or_tag(self._branch_path):
+        if branch_name == "" and layout.is_branch_or_tag(self._branch_path):
             return self._branch_path
         try:
-            if branch_name is not None:
-                branch_name = osutils.safe_utf8(branch_name)
+            branch_name = osutils.safe_utf8(branch_name)
             return layout.get_branch_path(branch_name, self._branch_path)
         except NoCustomBranchPaths:
-            if branch_name is None:
+            if branch_name == "":
                 return self._branch_path
             else:
                 raise errors.NoColocatedBranchSupport(layout)
 
-    def create_branch(self, branch_name=None, repository=None, mapping=None,
+    def create_branch(self, name=None, repository=None, mapping=None,
             lossy=False, append_revisions_only=None):
         """See ControlDir.create_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
@@ -466,7 +474,7 @@ class SvnRemoteAccess(ControlDir):
             if mapping is None:
                 mapping = repository.get_mapping()
 
-            relpath = self._determine_relpath(branch_name).strip("/")
+            relpath = self._determine_relpath(name).strip("/")
             if relpath == "":
                 if repository.get_latest_revnum() > 0:
                     # Bail out if there are already revisions in this repository
@@ -497,8 +505,13 @@ class SvnRemoteAccess(ControlDir):
         self.open_branch()
         return None
 
+    def set_branch_reference(self, target_branch, name=None):
+        # Sorry, no branch references.
+        raise errors.IncompatibleFormat(target_branch._format, self._format)
+
     def open_branch(self, name=None, unsupported=True, ignore_fallbacks=False,
-            mapping=None, branch_path=None, repository=None, revnum=None):
+            mapping=None, branch_path=None, repository=None, revnum=None,
+            possible_transports=None):
         """See ControlDir.open_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         if branch_path is None:
@@ -585,11 +598,15 @@ class SvnRemoteAccess(ControlDir):
         return self._config
 
     def list_branches(self):
+        return self.get_branches().values()
+
+    def get_branches(self):
         repos = self.find_repository()
         layout = repos.get_layout()
-        branches = []
+        branches = {}
         for project, bp, nick, has_props, revnum in layout.get_branches(repos,
                 repos.get_latest_revnum()):
-            branches.append(self.open_branch(branch_path=bp, repository=repos))
+            b = self.open_branch(branch_path=bp, repository=repos)
+            branches[b.name] = b
         return branches
 
