@@ -38,10 +38,7 @@ from bzrlib.repository import (
     RepositoryFormat,
     needs_read_lock,
     )
-try:
-    from bzrlib.revisiontree import InventoryRevisionTree
-except ImportError:# bzr < 2.4
-    from bzrlib.revisiontree import RevisionTree as InventoryRevisionTree
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.revision import (
     NULL_REVISION,
     ensure_null,
@@ -64,6 +61,7 @@ from bzrlib.plugins.svn import (
 from bzrlib.plugins.svn.branchprops import (
     PathPropertyProvider,
     )
+from bzrlib.plugins.svn.config import SvnRepositoryStack
 from bzrlib.plugins.svn.fileids import (
     CachingFileIdMapStore,
     FileIdMapStore,
@@ -340,6 +338,20 @@ class SubversionRepositoryCheckResult(branch.BranchCheckResult):
                 self.invalid_text_revisions += 1
 
 
+_optimizers_registered = False
+def lazy_register_optimizers():
+    """Register optimizers for fetching between svn and bzr repositories.
+    """
+    global _optimizers_registered
+    if _optimizers_registered:
+        return
+    from bzrlib.repository import InterRepository
+    from bzrlib.plugins.svn import push, fetch
+    _optimizers_registered = True
+    InterRepository.register_optimiser(fetch.InterFromSvnRepository)
+    InterRepository.register_optimiser(push.InterToSvnRepository)
+
+
 class SvnRepository(ForeignRepository):
     """
     Provides a simplified interface to a Subversion repository
@@ -349,7 +361,6 @@ class SvnRepository(ForeignRepository):
     chk_bytes = None
 
     def __init__(self, some_dir, transport, branch_path=None):
-        from bzrlib.plugins.svn import lazy_register_optimizers
         lazy_register_optimizers()
         self.vcs = foreign_vcs_svn
         _revision_store = None
@@ -474,10 +485,6 @@ class SvnRepository(ForeignRepository):
     def add_fallback_repository(self, basis_url):
         raise bzr_errors.UnstackableRepositoryFormat(self._format, self.base)
 
-    def revision_graph_can_have_wrong_parents(self):
-        # DEPRECATED in bzr 2.4
-        return False
-
     def pack(self, hint=None, clean_obsolete_packs=False):
         try:
             local_path = self.transport.local_abspath(".")
@@ -641,7 +648,7 @@ class SvnRepository(ForeignRepository):
             use_revprops = False
             use_fileprops = mapping.can_use_fileprops
         if (use_fileprops and
-            not target_config.get_allow_metadata_in_fileprops()):
+            not target_config.get('allow_metadata_in_file_properties')):
             raise errors.RequiresMetadataInFileProps()
         return (use_revprops, use_fileprops)
 
@@ -717,7 +724,7 @@ class SvnRepository(ForeignRepository):
     def store_layout(self, layout):
         """Permanantly store the layout for this repository."""
         self.set_layout(layout)
-        self.get_config().set_layout(layout)
+        self.get_config_stack().set('layout', str(layout))
         self._layout_source = LAYOUT_SOURCE_CONFIG
 
     def get_layout(self):
@@ -733,27 +740,29 @@ class SvnRepository(ForeignRepository):
             self._layout_source = LAYOUT_SOURCE_MAPPING_MANDATED
             self._layout = self.get_mapping().get_mandated_layout(self)
         if self._layout is None:
-            layoutname = self.get_config().get_layout()
+            config_stack = self.get_config_stack()
+            layoutname = config_stack.get('layout')
             if layoutname is not None:
                 self._layout_source = LAYOUT_SOURCE_CONFIG
                 self._layout = layout.layout_registry.get(layoutname)()
         if self._layout is None:
-            branches = self.get_config().get_branches()
-            tags = self.get_config().get_tags()
+            config_stack = self.get_config_stack()
+            branches = config_stack.get('branches')
+            tags = config_stack.get('tags')
             if branches is not None:
                 self._layout_source = LAYOUT_SOURCE_CONFIG
                 self._layout = WildcardLayout(branches, tags or [])
         if self._layout is None:
             if self._guessed_appropriate_layout is None:
-                self._find_guessed_layout(self.get_config())
+                self._find_guessed_layout(self.get_config_stack())
             self._layout_source = LAYOUT_SOURCE_GUESSED
             self._layout = self._guessed_appropriate_layout
         return self._layout, self._layout_source
 
-    def _find_guessed_layout(self, config):
+    def _find_guessed_layout(self, config_stack):
         # Retrieve guessed-layout from config and see if it accepts
         # self._hinted_branch_path
-        layoutname = config.get_guessed_layout()
+        layoutname = config_stack.get('guessed-layout')
         if layoutname is not None:
             config_guessed_layout = layout.layout_registry.get(layoutname)()
             if (self._hinted_branch_path is None or
@@ -770,7 +779,7 @@ class SvnRepository(ForeignRepository):
             self._guessed_appropriate_layout) = repository_guess_layout(self,
                     revnum, self._hinted_branch_path)
         if self._guessed_layout != config_guessed_layout and revnum > 200:
-            self.get_config().set_guessed_layout(self._guessed_layout)
+            config_stack.set('guessed-layout', self._guessed_layout)
 
     def get_guessed_layout(self):
         """Retrieve the layout bzr-svn deems most appropriate for this repo.
@@ -781,13 +790,9 @@ class SvnRepository(ForeignRepository):
         if self._guessed_layout is None:
             self._guessed_layout = self.get_mapping().get_guessed_layout(self)
         if self._guessed_layout is None:
-            self._find_guessed_layout(self.get_config())
+            self._find_guessed_layout(self.get_config_stack())
         return self._guessed_layout
 
-    def _warn_if_deprecated(self, branch=None): # for bzr < 2.4
-        # This class isn't deprecated
-        pass
-
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.base)
 
@@ -1086,6 +1091,9 @@ class SvnRepository(ForeignRepository):
     def get_config(self):
         return self.bzrdir.get_config()
 
+    def get_config_stack(self):
+        return SvnRepositoryStack(self)
+
     def has_signature_for_revision_id(self, revision_id):
         """Check whether a signature exists for a particular revision id.
 
@@ -1218,7 +1226,7 @@ class SvnRepository(ForeignRepository):
         if branch is None:
             raise Exception("branch option is required for "
                             "SvnRepository.get_commit_builder")
-        append_revisions_only = branch.get_config().get_append_revisions_only()
+        append_revisions_only = branch.get_config_stack().get('append_revisions_only')
         if append_revisions_only is None:
             append_revisions_only = True
         base_revmeta, base_mapping = branch.last_revmeta(skip_hidden=False)
