@@ -18,6 +18,10 @@
 import os
 import sys
 
+from bzrlib.controldir import (
+    ControlDir,
+    format_registry,
+    )
 import bzrlib.gpg
 from bzrlib import version_info as bzrlib_version
 from bzrlib.repository import Repository
@@ -27,7 +31,6 @@ from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.plugins.svn.convert import load_dumpfile
 from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
-    TrunkLayout,
     )
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
@@ -465,7 +468,8 @@ Node-copyfrom-path: x
         Sets up BZR_EDITOR so that if an editor is spawned it will run a
         script that just adds a known message to the start of the file.
         """
-        f = file('fed.py', 'wb')
+        path = os.path.join(self.test_dir, 'fed.py')
+        f = file(path, 'wb')
         f.write('#!%s\n' % sys.executable)
         f.write("""\
 # coding=utf-8
@@ -489,11 +493,11 @@ if len(sys.argv) == 2:
 "%s" fed.py %%1
 """ % sys.executable)
             f.close()
-            os.environ['BZR_EDITOR'] = 'fed.bat'
+            self.overideEnv('BZR_EDITOR', 'fed.bat')
         else:
             # [non-win32] make python script executable and set BZR_EDITOR
-            os.chmod('fed.py', 0755)
-            os.environ['BZR_EDITOR'] = './fed.py'
+            os.chmod(path, 0755)
+            self.overrideEnv('BZR_EDITOR', path)
 
     def test_set_branching_scheme_local(self):
         self.make_fake_editor()
@@ -603,11 +607,10 @@ if len(sys.argv) == 2:
         svn_url = self.make_repository('d')
 
         self.run_bzr('svn-import --format 1.9-rich-root %s dc' % svn_url)
+        cd = ControlDir.open('dc')
         self.assertEquals(
-            "Shared repository (format: 1.14-rich-root)\n"
-            "Location:\n"
-            "  shared repository: dc\n",
-            self.run_bzr('info dc')[0])
+            cd.open_repository()._format,
+            format_registry.make_bzrdir('1.9-rich-root').repository_format)
 
     def test_svn_layout(self):
         svn_url = self.make_repository('d')
@@ -622,7 +625,7 @@ if len(sys.argv) == 2:
             'Branch path: trunk\n'
             'Tag container directory: tags\n'
             'Branch container directory: branches\n'
-            'Push merged revisions: None\n' % svn_url,
+            'Push merged revisions: False\n' % svn_url,
                 self.run_bzr('svn-layout %s' % svn_url)[0])
 
     def test_svn_branches(self):
@@ -671,3 +674,49 @@ if len(sys.argv) == 2:
         self.commit_something(tree.branch.base)
         self.run_bzr('log -v d', retcode=0)
         self.run_bzr('log -v dc', retcode=0)
+
+    def test_svn_import_colocated(self):
+        svn_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(svn_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(svn_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir("branches/somebranch", "trunk")
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/release-1.0", "trunk")
+        dc.close()
+
+        self.run_bzr('init --development-colo dc')
+        self.run_bzr('svn-import d dc')
+        cd = ControlDir.open('dc')
+        self.assertEquals(set(["", "somebranch"]), set([b.name for b in cd.list_branches()]))
+
+
+class TestFixSvnAncestry(SubversionTestCase, ExternalBase):
+
+    def setUp(self):
+        ExternalBase.setUp(self)
+        self._init_client()
+
+    def tearDown(self):
+        ExternalBase.tearDown(self)
+
+    def test_smoke(self):
+        svn_tree = self.make_svn_branch_and_tree("d", "dc")
+        svn_revid = svn_tree.commit('some svn commit')
+
+        bzr_tree = svn_tree.bzrdir.sprout("e").open_workingtree()
+        bzr_revid = bzr_tree.commit('another commit')
+        self.assertEquals(
+            set([bzr_revid, svn_revid]),
+            set(bzr_tree.branch.repository.all_revision_ids()))
+
+        self.run_bzr("fix-svn-ancestry -d e d")
+
+        self.assertEquals(
+            set([bzr_revid, svn_revid]),
+            set(Repository.open("e").all_revision_ids()))
+
