@@ -26,6 +26,7 @@ import stat
 import subvertpy
 
 from subvertpy import (
+    ERR_BAD_FILENAME,
     ERR_WC_UNSUPPORTED_FORMAT,
     properties,
     )
@@ -124,21 +125,11 @@ from bzrlib.plugins.svn.tree import (
     SubversionTree,
     )
 
-# Compatibility with bzr < 2.5
-get_transport_from_path = getattr(_mod_transport, "get_transport_from_path",
-        _mod_transport.get_transport)
-
-# Compatibility with subvertpy < 0.8.6
-ERR_BAD_FILENAME = getattr(subvertpy, "ERR_BAD_FILENAME", 125001)
-
 # Compatibility with subvertpy < 0.8.9
 ERR_WC_NODE_KIND_CHANGE = getattr(subvertpy, "ERR_WC_NODE_KIND_CHANGE", 155018)
 
 
-try:
-    from bzrlib.controldir import Converter
-except ImportError:
-    from bzrlib.bzrdir import Converter
+from bzrlib.controldir import Converter
 from bzrlib.controldir import (
     ControlDirFormat,
     ControlDir,
@@ -1565,6 +1556,8 @@ class SvnWorkingTreeDirFormat(ControlDirFormat):
         except subvertpy.SubversionException, (_, num):
             if num in (subvertpy.ERR_RA_LOCAL_REPOS_OPEN_FAILED,):
                 raise NoSvnRepositoryPresent(transport.base)
+            if num in (subvertpy.ERR_WC_NOT_DIRECTORY,):
+                raise NotBranchError(path=transport.base)
             raise
 
     def get_format_string(self):
@@ -1697,7 +1690,7 @@ class SvnCheckout(ControlDir):
         return wt.branch.create_checkout(path, lightweight=True,
             revision_id=revision_id).bzrdir
 
-    def open_workingtree(self, _unsupported=False, recommend_upgrade=False):
+    def open_workingtree(self, unsupported=False, recommend_upgrade=False):
         wt_format = SvnWorkingTreeFormat(self._format.version)
         ret = SvnWorkingTree(self, wt_format, self.local_path, self.entry)
         try:
@@ -1748,7 +1741,7 @@ class SvnCheckout(ControlDir):
         raise NotImplementedError(self.create_branch)
 
     def open_branch(self, name=None, unsupported=True, ignore_fallbacks=False,
-            mapping=None, revnum=None):
+            mapping=None, revnum=None, possible_transports=None):
         """See ControlDir.open_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         repos = self._find_repository()
@@ -1759,7 +1752,8 @@ class SvnCheckout(ControlDir):
             return SvnBranch(repos, self.get_remote_bzrdir(),
                 self.get_branch_path(), mapping, revnum=revnum)
         except RepositoryRootUnknown:
-            return self.get_remote_bzrdir().open_branch(revnum=revnum)
+            return self.get_remote_bzrdir().open_branch(revnum=revnum,
+                possible_transports=possible_transports)
         except subvertpy.SubversionException, (_, num):
             if num == subvertpy.ERR_WC_NOT_DIRECTORY:
                 raise NotBranchError(path=self.base)
